---
obj: application
repo: https://github.com/eza-community/eza
website: https://eza.rocks
rev: 2024-05-27
---

# eza
[**eza**](https://eza.rocks/) is a modern replacement for the venerable file-listing command-line program `ls` that ships with Unix and Linux operating systems, giving it more features and better defaults. It uses colours to distinguish file types and metadata. It knows about symlinks, extended attributes, and [Git](../../dev/Git.md). And it’s **small**, **fast**, and just **one single binary**.

## Usage
Usage: `eza [OPTIONS] [FILES]`

### Options
#### Display Options
| Option                | Description                                                     |
| --------------------- | --------------------------------------------------------------- |
| `-1, --oneline`       | display one entry per line                                      |
| `-l, --long`          | display extended file metadata as a table                       |
| `-G, --grid`          | display entries as a grid (default)                             |
| `-x, --across`        | sort the grid across, rather than downwards                     |
| `-R, --recurse`       | recurse into directories                                        |
| `-T, --tree`          | recurse into directories as a tree                              |
| `-X, --dereference`   | dereference symbolic links when displaying information          |
| `-F, --classify=WHEN` | display type indicator by file names (always, auto, never)      |
| `--color=WHEN`        | when to use terminal colours (always, auto, never)              |
| `--color-scale`       | highlight levels of 'field' distinctly(all, age, size)          |
| `--color-scale-mode`  | use gradient or fixed colors in --color-scale (fixed, gradient) |
| `--icons=WHEN`        | when to display icons (always, auto, never)                     |
| `--no-quotes`         | don't quote file names with spaces                              |
| `--hyperlink`         | display entries as hyperlinks                                   |
| `--absolute`          | display entries with their absolute path (on, follow, off)      |
| `-w, --width COLS`    | set screen width in columns                                     |

#### Filter and Sorting Options
| Option                      | Description                                                                           |
| --------------------------- | ------------------------------------------------------------------------------------- |
| `-a, --all`                 | show hidden and 'dot' files. Use this twice to also show the '.' and '..' directories |
| `-A, --almost-all`          | equivalent to `--all`; included for compatibility with `ls -A`                        |
| `-d, --list-dirs`           | list directories as files; don't list their contents                                  |
| `-L, --level DEPTH`         | limit the depth of recursion                                                          |
| `-r, --reverse`             | reverse the sort order                                                                |
| `-s, --sort SORT_FIELD`     | which field to sort by                                                                |
| `--group-directories-first` | list directories before other files                                                   |
| `-D, --only-dirs`           | list only directories                                                                 |
| `-f, --only-files`          | list only files                                                                       |
| `-I, --ignore-glob GLOBS`   | glob patterns (pipe-separated) of files to ignore                                     |
| `--git-ignore`              | ignore files mentioned in `.gitignore`                                                |

Valid sort fields:
- name
- extension
- size
- type
- modified
- accessed
- created
- inode
- none

#### Long View Options
| Option                    | Description                                                                                                                 |
| ------------------------- | --------------------------------------------------------------------------------------------------------------------------- |
| `-b, --binary`            | list file sizes with binary prefixes                                                                                        |
| `-B, --bytes`             | list file sizes in bytes, without any prefixes                                                                              |
| `-g, --group`             | list each file's group                                                                                                      |
| `--smart-group`           | only show group if it has a different name from owner                                                                       |
| `-h, --header`            | add a header row to each column                                                                                             |
| `-H, --links`             | list each file's number of hard links                                                                                       |
| `-i, --inode`             | list each file's inode number                                                                                               |
| `-m, --modified`          | use the modified timestamp field                                                                                            |
| `-M, --mounts`            | show mount details (Linux and Mac only)                                                                                     |
| `-n, --numeric`           | list numeric user and group IDs                                                                                             |
| `-O, --flags`             | list file flags (Mac, BSD, and Windows only)                                                                                |
| `-S, --blocksize`         | show size of allocated file system blocks                                                                                   |
| `-t, --time FIELD`        | which timestamp field to list (modified, accessed, created)                                                                 |
| `-u, --accessed`          | use the accessed timestamp field                                                                                            |
| `-U, --created`           | use the created timestamp field                                                                                             |
| `--changed`               | use the changed timestamp field                                                                                             |
| `--time-style`            | how to format timestamps (default, iso, long-iso, full-iso, relative, or a custom style `+<FORMAT>` like `+%Y-%m-%d %H:%M`) |
| `--total-size`            | show the size of a directory as the size of all files and directories inside (unix only)                                    |
| `--no-permissions`        | suppress the permissions field                                                                                              |
| `-o, --octal-permissions` | list each file's permission in octal format                                                                                 |
| `--no-filesize`           | suppress the filesize field                                                                                                 |
| `--no-user`               | suppress the user field                                                                                                     |
| `--no-time`               | suppress the time field                                                                                                     |
| `--stdin`                 | read file names from stdin, one per line or other separator specified in environment                                        |
| `--git`                   | list each file's [Git](../../dev/Git.md) status, if tracked or ignored                                                      |
| `--no-git`                | suppress [Git](../../dev/Git.md) status (always overrides `--git`, `--git-repos`, `--git-repos-no-status`)                  |
| `--git-repos`             | list root of git-tree status                                                                                                |
| `-@, --extended`          | list each file's extended attributes and sizes                                                                              |
| `-Z, --context`           | list each file's security context                                                                                           |
